include (jslib_dir);
include (jslib_dirutils);
include (jslib_fileutils);
include (jslib_file);
include ("chrome://mozimage/content/mozimage_prefs.js");
include ("chrome://global/content/strres.js");

const XMLNS = "http://www.w3.org/XML/1998/namespace";

/*
**
** Private functions
**
*/

var mozImageBundle = srGetStrBundle("chrome://mozimage/locale/mozimage.properties");
var numOfLinks = 0;
var emptyingList = false;
var preloadedImages = null;

function showFile(imageURL)
{
  try {

    var currentDocument = top.getBrowser().contentDocument;
    var currentWindow = top.getBrowser();
    currentDocument.location.href = imageURL;

  } catch(e) { alert(e); }
}

function resize(ratio)
{
  var contentDocument = top.getBrowser().contentDocument;
  if (contentDocument.images.length == 1)
  {
    if (prefs.getAutoSize())
      autosize_click();

    var image = contentDocument.images[0];
    imageWidth = image.width;
    imageHeight = image.height;
    image.width = imageWidth * ratio;
    image.height = imageHeight * ratio;
  }
}

function autoSize()
{
  var contentDocument = top.getBrowser().contentDocument;
  
  if (contentDocument.images.length == 1)
  {
    var image = contentDocument.images[0];
  
    var boxWidth = contentDocument.width;
    var boxHeight = contentDocument.height;

    var xratio = image.width / boxWidth;
    var yratio = image.height / boxHeight;
    var ratio = (xratio > yratio) ? xratio : yratio;

    imageWidth = image.width / ratio;
    imageHeight = image.height / ratio;
  
    image.width = imageWidth;
    image.height = imageHeight;
  }
}

function updateDescription()
{
  var image = document.getElementById("file-listbox").selectedItem.lastChild.firstChild;
  var info = document.getElementById("info-label");
  var newImage = new Image();
  newImage.setAttribute("id", "thepreviewimage");
  newImage.src = image.src;
  info.value = mozImageBundle.formatStringFromName("sizeInfo", [newImage.width, newImage.height], 2);
}

function fileNameCompare(name1,name2)
{

   if(name1.leaf.toLowerCase() < name2.leaf.toLowerCase())
      return -1;
   if(name1.leaf.toLowerCase() > name2.leaf.toLowerCase())
      return 1;
   return 0;
}
 
function fileNameNumCompare(name1,name2) 
{ 
  // work with filenames such as: pic1.jpg, pic2.jpg, ..., pic9.jpg, pic10.jpg etc. 
  var filenameRe = /^(\D*)(\d*)(.*)$/; 
  var fn1 = name1.leaf.toLowerCase(); 
  var fn2 = name2.leaf.toLowerCase(); 
  while (fn1 != "" || fn2 != "") { 
    var filenameComponents1 = filenameRe.exec(fn1); 
    var filenameComponents2 = filenameRe.exec(fn2); 
    if (filenameComponents1[1] < filenameComponents2[1]) return -1; 
    if (filenameComponents1[1] > filenameComponents2[1]) return 1; 
    var rv = filenameComponents1[2] - filenameComponents2[2]; 
    if (rv != 0) return rv; 
    fn1 = filenameComponents1[3]; 
    fn2 = filenameComponents2[3]; 
  } 
  return 0; 
} 
 
function fileCompare(name1,name2) 
{ 
  var method = prefs.getOrderBy(); 
  var descending = (prefs.getDescending() ? -1 : 1); 
  switch (method) 
  { 
    case "name" : 
      return descending*fileNameCompare(name1,name2); 
      break; 
    case "nume" : 
      return descending*fileNameNumCompare(name1,name2); 
      break; 
    case "date" : 
      return descending*(name1.dateModified.getTime() - name2.dateModified.getTime()); 
      break; 
    case "size" : 
      return descending*(name1.size - name2.size); 
      break; 
    default : 
      return descending*fileNameCompare(name1,name2); 
  } 
  return 0; 
} 
 
function fillListBox(aLocation) 
{ 
  var info = document.getElementById("info-label"); 
  var loadingStr = mozImageBundle.formatStringFromName("loading", [], 0); 
  var readyStr = mozImageBundle.formatStringFromName("ready", [], 0); 
   
  info.value = loadingStr; 
 
  if (aLocation.substr(0, 7) == "http://") 
    initializeSearch(aLocation); 
  else 
    directorySearch(aLocation); 
     
  info.value = readyStr; 
} 
 
function initializeSearch(url) 
{ 
  var fullpath = document.getElementById("fullpath-text"); 
  var aBrowser = document.getElementById("html-browser"); 
   
  aBrowser.addEventListener("load", htmlSearch, true); 
   
  numOfLinks = 0; 
  fullpath.value = url; 
  aBrowser.setAttribute('src', url); 
} 
 
function htmlSearch() 
{

  var dirlistbox = document.getElementById("directory-listbox"); 
  var filelistbox = document.getElementById("file-listbox"); 
  var aBrowser = document.getElementById("html-browser"); 
  var doc = aBrowser.contentDocument; 
  preloadedImages = new Array(); 
  if (doc.links.length > numOfLinks) 
  { 
    emptyList(); 
    numOfLinks = doc.links.length; 
    for (var i = 0 ; i < doc.links.length ; i++) 
    { 
      if (supportedLinkType(doc.links[i])) 
      { 
        filelistbox.appendChild(getImageLinkItem(doc.links[i])); 
        // fill an image object to force the cache  
        // to be used 
        if (prefs.getEnableCache()) 
        { 
          preloadedImages[i] = new Image(); 
          preloadedImages[i].src = doc.links[i].href; 
        } 
      } 
      else if (doc.links[i].href.substr(0, 7) == "http://") 
        dirlistbox.appendChild(getDirectoryItem(doc.links[i])); 
    } 
  } 
}     
 
function directorySearch(adir) 
{ 
  var dirnotfound = mozImageBundle.formatStringFromName("dirnotfound", [], 0); 
  var dirlistbox = document.getElementById("directory-listbox"); 
  var filelistbox = document.getElementById("file-listbox"); 
  var d = new Dir(adir); 
  var dirList = d.readDir(); 
  if (dirList == null) 
  { 
    alert(dirnotfound); 
    return; 
  } 
   
  var fullpath = document.getElementById("fullpath-text"); 
 
  var dirs = new Array(); 
  var files = new Array(); 
  var i = 0; 
  emptyList(); 
  fullpath.value = adir; 
   
  for (i = 0 ; i < dirList.length ; i++)  
    dirList[i].isDir() ?  dirs.push(dirList[i]) : files.push(dirList[i]); 
 
  dirs.sort(fileNameCompare); 
  files.sort(fileCompare); 
 
  dirList = new Array(); 
 
  for (i = 0 ; i < dirs.length ; i++) 
  { 
    var listItem = dirlistbox.appendItem(dirs[i].leaf, ''); 
    // add the attributes to show right icon 
    listItem.setAttribute("class", "listitem-iconic");
    listItem.setAttribute("value", "directory"); 
  } 
 
  for (i = 0 ; i < files.length ; i++) 
    if (supportedType(files[i])) 
      filelistbox.appendChild(getFileItem(files[i].leaf)); 
    if (prefs.getEnableCurrent())
      prefs.setHomeDir(adir); 
} 
 
function getDirectoryItem(linkItem) 
{ 
  var fullpath = document.getElementById("fullpath-text"); 
  var listItem = document.createElement('listitem'); 
  var listCell = document.createElement('listcell'); 
  var fileLabel = document.createElement('label'); 
  fileLabel.setAttribute('value', linkItem.href); 
  fileLabel.setAttribute('crop', 'center'); 
  fileLabel.setAttribute('flex', '1'); 
 
  listCell.appendChild(fileLabel); 
  listItem.appendChild(listCell); 
  listItem.setAttribute("class", "listitem-iconic"); 
  listItem.setAttribute("value", "directory"); 
  // For compatibility with directoy viewer 
  listItem.setAttribute("label", linkItem.href); 
  return(listItem); 
} 
 
function getAbsoluteURL(url, node) 
{ 
  if (!url || !node) 
    return ""; 
 
  var urlArr = new Array(url); 
  var doc = node.ownerDocument; 
 
  if (node.nodeType == Node.ATTRIBUTE_NODE) 
    node = node.ownerElement; 
  
  while (node && node.nodeType == Node.ELEMENT_NODE)  
  { 
    if (node.getAttributeNS(XMLNS, "base") != "") 
      urlArr.unshift(node.getAttributeNS(XMLNS, "base")); 
 
    node = node.parentNode; 
  } 
 
  // Look for a <base>. 
  var baseTags = doc.getElementsByTagName("base"); 
  if (baseTags && baseTags.length)  
  { 
    urlArr.unshift(baseTags[baseTags.length - 1].getAttribute("href")); 
  } 
 
  // resolve everything from bottom up, starting with document location 
  var ioService = Components.classes["@mozilla.org/network/io-service;1"] 
                            .getService(Components.interfaces.nsIIOService); 
  var URL = ioService.newURI(doc.location.href, null, null); 
  for (var i=0; i<urlArr.length; i++)  
  { 
    URL.spec = URL.resolve(urlArr[i]); 
  } 
 
  return URL.spec; 
} 
 
function getImageLinkItem(linkItem) 
{ 
  var fullpath = document.getElementById("fullpath-text"); 
  var listItem = document.createElement('listitem'); 
  var listCell = document.createElement('listcell'); 
  var fileLabel = document.createElement('label'); 
 
  fileLabel.setAttribute('value', linkItem.href); 
  fileLabel.setAttribute('crop', 'center'); 
  fileLabel.setAttribute('flex', '1'); 
 
  if (prefs.getThumbSize() > 0) 
  { 
    if (linkItem.firstChild.nodeName.toUpperCase() == 'IMG' || prefs.getForceHttpTumb()) 
    { 
      var tumbURL = ""; 
      var fileImage = document.createElement('image'); 
      if (linkItem.firstChild.nodeName.toUpperCase() == 'IMG') 
      { 
        var image = linkItem.firstChild; 
        tumbURL = getAbsoluteURL(image.src, image); 
      } 
      if (tumbURL == "") 
        tumbURL = linkItem.href; 
      fileImage.setAttribute('src', tumbURL); 
      fileImage.setAttribute('width', prefs.getThumbSize()); 
      fileImage.setAttribute('height', prefs.getThumbSize()); 
      listCell.appendChild(fileImage); 
    } 
  } 
  listCell.appendChild(fileLabel); 
  listItem.appendChild(listCell); 
  listItem.setAttribute('context', 'filelist-popup'); 
  return(listItem);
}

function getFileItem(fileName)
{
  var fullpath = document.getElementById("fullpath-text");
  var fileUtil = new FileUtils();
  var fullFileName = pathToURI(fileUtil.append(fullpath.value, fileName));

  var listItem = document.createElement('listitem');
  var listCell = document.createElement('listcell');
  var fileLabel = document.createElement('label');
  
  fileLabel.setAttribute('value', fileName);
  
  if (prefs.getThumbSize() > 0)
  {
    var fileImage = document.createElement('image');
    fileImage.setAttribute('src', fullFileName);
    fileImage.setAttribute('width', prefs.getThumbSize());
    fileImage.setAttribute('height', prefs.getThumbSize());
    listCell.appendChild(fileImage);
  }
  
  listCell.appendChild(fileLabel);
  listItem.appendChild(listCell);
  listItem.setAttribute('context', 'filelist-popup');
  return(listItem);
}

function supportedLinkType(linkItem)
{
  var result = false;
  var extList = prefs.getExtList();
  for (var i = 0 ; i < extList.length ; i++)
  {
    if (extList[i].toUpperCase() == linkItem.href.substr(-3).toUpperCase())
      result = true;
  }
  return(result);
}

function supportedType(file)
{
  var result = false;
  var extList = prefs.getExtList();
  for (var i = 0 ; i < extList.length ; i++)
  {
    if (extList[i].toUpperCase() == file.ext.toUpperCase())
      result = true;
  }    
  return(result);
}

function emptyList()
{
  emptyingList = true;
  var dirlistbox = document.getElementById("directory-listbox");
  var filelistbox = document.getElementById("file-listbox");
  var item;
  
  var i = dirlistbox.getRowCount();
  while (i > 0)
  {
    i--;
    item = dirlistbox.getItemAtIndex(i);
    dirlistbox.removeChild(item);
  }
  i = filelistbox.getRowCount();
  while (i > 0)
  {
    i--;
    item = filelistbox.getItemAtIndex(i);
    filelistbox.removeChild(item);
  }
  emptyingList = false;
}

function doSlideShow()
{
  if ( prefs.getSlideshow() )
  {
    next_click();
    setTimeout(doSlideShow, prefs.getDelay());
  }
}

function selectItemByName(itemName)
{
  var fileListBox = document.getElementById("file-listbox");
  var numOfRows = fileListBox.getRowCount();
  
  for(var i = 0 ; i < numOfRows ; i++)
  {
    item = fileListBox.getItemAtIndex(i);
    fileListBox.scrollToIndex(i);
    if ( item.lastChild.lastChild.value == itemName )
    {
      fileListBox.selectItem(item);
    }

  }
  fileListBox.ensureIndexIsVisible(fileListBox.selectedIndex);
}

function parseCommandLine()
{
  var cmdLineService = Components.classes["@mozilla.org/appshell/commandLineService;1" ].getService( Components.interfaces.nsICmdLineService );
  var path = cmdLineService.URLToLoad;

  if (path != null && path != "")
  {
    var file = new File(path);
    if (file.isDir())
      fillListBox(file.path)
    else
    {
      fillListBox(file.parent.path);
      selectItemByName(file.leaf);
    }
  }
}

function buildOpenWithMenu()
{
  
  var editorsName = prefs.getEditorsName();
  for (var i = 1 ; i <= 4 ; i++)
  {
    var openwith = document.getElementById("openwith" + i + "-menu");
    openwith.label = mozImageBundle.formatStringFromName("openwith", [editorsName[i-1]], 1);
    openwith.hidden = (editorsName[i-1] == "");
  }
}

function buildMacroMenu()
{
  var macroName = prefs.getMacroName();
  for (var i = 0 ; i < 10 ; i++)
  {
    var macromenu = document.getElementById("macro" + i + "-menu");
    macromenu.label = macroName[i];
    macromenu.hidden = (macroName[i] == "");
  }
}

function pathToURI(aPath)
{
  const JS_URIFIX = new Components.Constructor("@mozilla.org/docshell/urifixup;1", "nsIURIFixup");
  var uriFixup = new JS_URIFIX();

  var fixedURI = uriFixup.createFixupURI(aPath, 0);
  return(fixedURI.spec);
}

function clearCache() 
{
  var classID = Components.classes["@mozilla.org/network/cache-service;1"];
  var cacheService = classID.getService(Components.interfaces.nsICacheService);
  cacheService.evictEntries(Components.interfaces.nsICache.STORE_ON_DISK);
  cacheService.evictEntries(Components.interfaces.nsICache.STORE_IN_MEMORY);
}

function getParams(commandLine, imagePath)
{
  var params = new Array();
  var index = 0;
  var quote = false;
  var i = 0;
  params[index] = "";
  for (i = 0 ; i < commandLine.length ; i++ )
  {
    if (commandLine[i] == "'")
      quote = !quote;
    else if (commandLine[i] == ' ' && !quote)
    {
      index++;
      params[index] = "";
    }
    else
      params[index] = params[index] + commandLine[i];
  }
  for (i = 0 ; i < params.length ; i++ )
  {
    if (params[i] == '%f')
      params[i] = imagePath;
  }
  return(params);
}

function saveBookmarks()
{
  var bookmarklistbox = document.getElementById("bookmark-listbox");
  var dirUtil = new DirUtils();
  var fileUtil = new FileUtils();
  var fileName = fileUtil.append(dirUtil.getPrefsDir(), 'mozimage-bookmarks.txt');
  var file = new File(fileName);
  var anItem = null;
  file.open('w');
  for (var i = 0 ; i < bookmarklistbox.getRowCount() ; i++)
  {
    anItem = bookmarklistbox.getItemAtIndex(i);
    file.write(anItem.label + '\n');
    file.write(anItem.value + '\n');
  }
  file.close();
}

function loadBookmarks()
{
  try {
  
  var bookmarklistbox = document.getElementById("bookmark-listbox");
  var dirUtil = new DirUtils();
  var fileUtil = new FileUtils();
  var fileName = fileUtil.append(dirUtil.getPrefsDir(), 'mozimage-bookmarks.txt');
  var file = new File(fileName);

  if (!file.exists())
  {
    file.create();
    file.close();
  }
  else
  {
    file.open("r");
    var bookmarkName = "";
    var bookmarkValue = "";
    while(!file.EOF) 
    {
      bookmarkName = file.readline();
      if(!file.EOF) 
        bookmarkValue = file.readline();

      if (bookmarkName != '')
      {
        var listItem = bookmarklistbox.appendItem(bookmarkName, '');
        // add the attributes to show right icon
        listItem.setAttribute("class", "listitem-iconic");
        listItem.setAttribute("value", bookmarkValue);
        listItem.setAttribute("ondblclick", "bookmark_select(event);");
      }
    }
    file.close();
  }
  
  } catch(e) { alert(e); }
}

function bookmarkup_click()
{
  var bookmarklistbox = document.getElementById("bookmark-listbox");
  var item = bookmarklistbox.selectedItem;

  // firstChild does't return a ListItem
  if (item != bookmarklistbox.getItemAtIndex(0))
  {
    prevItem = item.previousSibling;
    bookmarklistbox.removeChild(item);
    bookmarklistbox.insertBefore(item, prevItem);
    bookmarklistbox.selectItem(item);
    saveBookmarks();
  }
}

function bookmarkdown_click()
{
  var bookmarklistbox = document.getElementById("bookmark-listbox");
  var item = bookmarklistbox.selectedItem;
  if (item != bookmarklistbox.lastChild)
  {
    nextItem = item.nextSibling.nextSibling;
    bookmarklistbox.removeChild(item);
    bookmarklistbox.insertBefore(item, nextItem);
    bookmarklistbox.selectItem(item);
    saveBookmarks();
  }
}

function save(url, fileName) 
{ 
  var ioservice = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
  var uri = ioservice.newURI(url, null, null); 
  var persist = Components.classes["@mozilla.org/embedding/browser/nsWebBrowserPersist;1"].createInstance(Components.interfaces.nsIWebBrowserPersist);
  var targetFile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile); 

  targetFile.initWithPath(fileName); 

  if(!targetFile.exists()) 
  { 
    targetFile.create(0x00,0644); 
  } 
  persist.saveURI(uri, null, null, null, null, targetFile); 
}

function baseNameFromUrl(url)
{
  var nsIURL = Components.interfaces.nsIURL;
  var urlComp = Components.classes['@mozilla.org/network/standard-url;1'].createInstance(nsIURL);
  urlComp.spec = url;
  return(urlComp.fileName);
}

/*
**
** Event handler
**
*/

function launchmozimage_click(getLink)
{
  // mozimage_url is copied into the sidebar object because
  // if the toggleSidebar function is called for the first
  // time and create the sidebar, when the code reach the
  // call to fillListBox the sidebar probably will not
  // exist yet. So the mozimage_url is copied in the sidebar
  // object and during the window_load is read and used.
  
  toggleSidebar('viewMozImageSidebar', true);
  var sidebar = document.getElementById("sidebar");
  var isMozImage = sidebar.getAttribute("src").indexOf("mozimage") > -1;
  sidebar.setAttribute('mozimage_url', gBrowser.contentWindow.location.href);
  if (getLink && sidebar && isMozImage)
    sidebar.contentWindow.fillListBox(gBrowser.contentWindow.location.href);
}

//
// Added for Mozilla Sidebar (tested In mozilla 1.7.7)
//

function launchmozimage_click_mozilla(getLink)
{
    addPanel(mozImageBundle.GetStringFromName("sidebarTitle"), "chrome://mozimage/content/mozimage.xul", "");
    //addPanel("MozImage SideBar", "http://www.google.com/", "");
    //toggleSidebar('viewMozImageSidebar', true);
    //var ref = top.document.getElementById("viewMozImageSidebar");
    //ref.setAttribute('mozimageLocation', gBrowser.contentWindow.location.href);
}

function window_load(event)
{
  try {
    var fullpath = document.getElementById("fullpath-text");
    var autoSizeButton = document.getElementById("autosize-button");
    var aBrowser = document.getElementById("html-browser");
    var sidebar = top.document.getElementById("sidebar");
    var startingUrl = sidebar.getAttribute("mozimage_url");
    
    autoSizeButton.setAttribute("checked", prefs.getAutoSize());

    buildOpenWithMenu();
    buildMacroMenu();
    loadBookmarks();
    
    window.onresize = image_resize;
    
    if (startingUrl != "" && fullpath.value != startingUrl) 
      fillListBox(startingUrl);
    else if (fullpath.value == "")
    {
      parseCommandLine();
      if (fullpath.value == "")
        fillListBox(prefs.getHomeDir());
    }

  } catch(e) { alert(e); }
}

function window_close(event)
{
  var aBrowser = document.getElementById("html-browser");
  prefs.setSlideshow(false);
  prefs.save();
}

function about_click()
{
  window.openDialog('chrome://mozimage/content/mozimage_about.xul', 'about', 'chrome,centerscreen,dialog,modal');
}

function options_click()
{
  try {
    prefs.save();
    window.openDialog('chrome://mozimage/content/mozimage_prefs.xul', mozImageBundle.GetStringFromName("optionsTitle"), 'chrome,centerscreen,dialog,modal', document.getElementById("fullpath-text"),  mozImageBundle.GetStringFromName("choosedir"));
    prefs.load();
    buildOpenWithMenu();
    var fullpath = document.getElementById("fullpath-text");
    fillListBox(fullpath.value);
  } catch(e) { alert(e); }
}

function macroconf_click()
{
  prefs.save();
  window.openDialog('chrome://mozimage/content/mozimage_macros.xul', 'Macros', 'chrome,centerscreen,dialog,modal');
  prefs.load();
  buildMacroMenu();
}

function exit_click()
{
  try {
    var fullpath = document.getElementById("fullpath-text");
    if (prefs.getEnableCurrent())
    { 
      prefs.setHomeDir(fullpath.value);
      prefs.save();
    }
  } catch (e) { alert (e); }
  window.close();
}

function image_resize()
{
  if (prefs.getAutoSize())
    autoSize();
}

function autosize_click()
{
  try {
    var autoSizeButton = document.getElementById("autosize-button");
    var mustAutosize = !prefs.getAutoSize();
  
    prefs.setAutoSize(mustAutosize);
    autoSizeButton.setAttribute('checked', mustAutosize);
    
    prefs.save();

    if (mustAutosize)
      autoSize()
    else
      top.getBrowser().reload();
  
  } catch (e) { alert (e); }
}

function zoomout_click()
{
  try {
    resize(1/prefs.getZoom());
  } catch(e) { alert(e); }
}

function zoomin_click()
{
  try {
    resize(prefs.getZoom());
  } catch(e) { alert(e); }
}

function edit_click()
{
  try {
    var chooseImageBefore = mozImageBundle.formatStringFromName("chooseimagebefore", [], 0);
    var filelistbox = document.getElementById("file-listbox");
    var selectedItem = filelistbox.selectedItem;

    if (selectedItem == null) {
      alert(chooseImageBefore);
      return;
    }

    var filenotfound = mozImageBundle.formatStringFromName("filenotfound", [], 0);
    var labelItem = selectedItem.lastChild.lastChild;
    var fileName = labelItem.value;
    var basePath = document.getElementById("fullpath-text");
    var fileUtil = new FileUtils();
    var imagePath = fileUtil.append(basePath.value, fileName);

    if (imagePath == null) {
      alert(chooseImageBefore);
      return;
    }

    var convertPath = prefs.getConvertPath();
    var editWindow = window.openDialog('chrome://mozimage/content/mozimage_edit.xul', 
                                     mozImageBundle.GetStringFromName("editTitle"), 
                                     'chrome,centerscreen,modal', 
                                     //document.getElementById("main-image"),
                                     //top.getBrowser().contentDocument.images[0],
                                     document.getElementById("file-listbox").selectedItem.lastChild.firstChild,
                                     mozImageBundle.formatStringFromName("saveas", [], 0), 
                                     document.getElementById("file-listbox"),
                                     document.getElementById("fullpath-text"),
                                     mozImageBundle.formatStringFromName("commandnotfound", [], 0),
                                     convertPath,
                                     document.getElementById("directory-listbox"));
                                     
  } catch(e) { alert(e); }
}

function slideshow_click()
{
  try {
    var slideshowButton = document.getElementById("slideshow-button");
    var slideshow = !prefs.getSlideshow();
  
    prefs.setSlideshow(slideshow);
    slideshowButton.setAttribute('checked', slideshow);

    doSlideShow();

  } catch(e) { alert(e); }
}

function image_click()
{
    //var image = document.getElementById("main-image");
     var image = document.getElementById("file-listbox").selectedItem.lastChild.firstChild;
  image.focus();
}

function clear_click()
{
  try {
    // Clear image
    //var image = document.getElementById("main-image");
     var image = document.getElementById("file-listbox").selectedItem.lastChild.firstChild;
    image.src = "";
    image.width = 0;
    image.height = 0;
    // and stop slide show
    var slideShowMenu = document.getElementById("slideshow-menu");
    slideShowMenu.setAttribute("checked", "false");
  } catch(e) { alert(e); }
}

function prior_click()
{
  var fileListBox = document.getElementById("file-listbox");
  var selectedIndex = fileListBox.selectedIndex;
  if (selectedIndex >= 1)
    fileListBox.selectedIndex = selectedIndex - 1;
  else
    fileListBox.selectedIndex =  fileListBox.getRowCount() - 1;
  fileListBox.ensureIndexIsVisible ( fileListBox.selectedIndex );
}

function next_click()
{
  var fileListBox = document.getElementById("file-listbox");
  var selectedIndex = fileListBox.selectedIndex;
  if (selectedIndex >= -1 && selectedIndex < fileListBox.getRowCount() - 1)
    fileListBox.selectedIndex = selectedIndex + 1;
  else
    fileListBox.selectedIndex = 0;
  fileListBox.ensureIndexIsVisible ( fileListBox.selectedIndex );
}

function refresh_click()
{
  try {
    var fullpath = document.getElementById("fullpath-text");
    var fileUtil = new FileUtils();
    if (fullpath.value)
    {
      var aPath = fullpath.value;
      //var file = new File(aPath);
      //var aParent = file.parent.path;
      fillListBox(aPath);
    }
  } catch(e) { alert(e); }
}

function up_click()
{
  try {
    var fullpath = document.getElementById("fullpath-text");
    var fileUtil = new FileUtils();
    if (fullpath.value)
    {
      var aPath = fullpath.value;
      var file = new File(aPath);
      var aParent = file.parent.path;
      fillListBox(aParent);
    }
  } catch(e) { }
}

function go_click(event)
{
  try {
    var fullpath = document.getElementById("fullpath-text");
    fillListBox(fullpath.value);
  } catch(e) { alert(e); }
}

function directory_select(event)
{
  if (!emptyingList)
  {
    var selectedItem = event.target.selectedItem;
    var basePath = document.getElementById("fullpath-text");
    var path = selectedItem.getAttribute("label");
    var fileUtil = new FileUtils();
    var url = "";

    if (path.substr(0, 7) == "http://")
      url = path;
    else
      url = fileUtil.append(basePath.value, path);
    // the fillListBox is delayed because probably during its execution it deletes
    // all directory-listbox elements. So, when the code return to this event
    // handler the listboxitem that fire the event don't exists no more and mozilla
    // returns an error.
    setTimeout("fillListBox('" + url.replace(/\\/g, "\\\\") + "')", 10);
  }
}

function bookmark_select(event)
{
  try {
    var selectedItem = event.target;
    var path = selectedItem.getAttribute("value");
    fillListBox(path);
  } catch(e) { alert(e); }

}

function addbookmark_click()
{
  var bookmarklistbox = document.getElementById("bookmark-listbox");
  var fullpath = document.getElementById("fullpath-text");
  var question = mozImageBundle.formatStringFromName("bookmarkadd", [], 0);
  var bookmarkName = prompt(question, fullpath.value);
  if (bookmarkName != null)
  {
    var listItem = bookmarklistbox.appendItem(bookmarkName, '');
    // add the attributes to show right icon
    listItem.setAttribute("class", "listitem-iconic");
    listItem.setAttribute("value", fullpath.value);
  }
  saveBookmarks();
}

function deletebookmark_click()
{

  var bookmarklistbox = document.getElementById("bookmark-listbox");
  var anItem = bookmarklistbox.currentItem;
  var question = mozImageBundle.formatStringFromName("bookmarkdelete", [anItem.label], 1);

  if ( confirm(question) )
  {
    bookmarklistbox.removeChild(anItem);  
  }
  saveBookmarks();
}

function file_select(event)
{
  if (!emptyingList)
  {
    var selectedItem = event.target.selectedItem;
    var basePath = document.getElementById("fullpath-text");
    var fileUtil = new FileUtils();
   
    var labelItem = selectedItem.lastChild.lastChild;
    var fileName = labelItem.value;
  
    if (fileName.substr(0, 7) != "http://")
      var uri = pathToURI(fileUtil.append(basePath.value, fileName));
    else
      uri = fileName;
    
    showFile(uri);
  }
}

function fullscreen_click()
{
  window.fullScreen = !window.fullScreen;

  var statusbar = document.getElementById("info-statusbar");
  var menubar = document.getElementById("menu-bar");
  var restoreButton = document.getElementById("restore-button");

  statusbar.hidden = window.fullScreen;
  menubar.hidden   = window.fullScreen;
  restoreButton.hidden = !window.fullScreen;
}

function openwith_click(index)
{
  var chooseImageBefore = mozImageBundle.formatStringFromName("chooseimagebefore", [], 0);
  var filelistbox = document.getElementById("file-listbox");
  var selectedItem = filelistbox.selectedItem;

  if (selectedItem == null) {
    alert(chooseImageBefore);
    return;
  }

  var labelItem = selectedItem.lastChild.lastChild;
  var fileName = labelItem.value;
  var filenotfound = mozImageBundle.formatStringFromName("filenotfound", [], 0);
  
  var basePath = document.getElementById("fullpath-text");
  var fileUtil = new FileUtils();
  var editorsPath = prefs.getEditorsPath();
  //var image = document.getElementById("main-image");
  var image = document.getElementById("file-listbox").selectedItem.lastChild.firstChild;
  
  var imagePath = fileUtil.append(basePath.value, fileName);
  
  try {
    var rv = fileUtil.spawn(editorsPath[index - 1], [imagePath]);
    if (rv==null)
      alert(filenotfound);
  } catch (e) { alert(e); }
}

function macro_click(index)
{
  var chooseImageBefore = mozImageBundle.formatStringFromName("chooseimagebefore", [], 0);
  var filenotfound = mozImageBundle.formatStringFromName("filenotfound", [], 0);
  var filelistbox = document.getElementById("file-listbox");
  var selectedItem = filelistbox.selectedItem;

  if (selectedItem == null) {
    alert(chooseImageBefore);
    return;
  }
  var labelItem = selectedItem.lastChild.lastChild;
  var fileName = labelItem.value;
  var basePath = document.getElementById("fullpath-text");
  var fileUtil = new FileUtils();
  var imagePath = fileUtil.append(basePath.value, fileName);

  var image = document.getElementById("file-listbox").selectedItem.lastChild.firstChild;
  var imageURL = image.src;

  var macroCode = prefs.getMacroCode();
  var commandLine = macroCode[index];
  var paramsList = getParams(commandLine, imagePath);
  var paramsOnly = new Array();
  var command = paramsList[0];

  for (var i = 1 ; i < paramsList.length ; i++)
    paramsOnly[i-1] = paramsList[i];

  try {
    var rv = fileUtil.execute(command, paramsOnly);
    if (rv==null)
      alert(filenotfound);
    } catch (e) {
      alert(e);
  }
  var thumbItem = selectedItem.lastChild.firstChild;
  if (thumbItem.nodeName == "image")
    thumbItem.src = "";
    
  image.src = "";
  clearCache();
  image.src = imageURL;
  
  if (thumbItem.nodeName == "image")
    thumbItem.src = imageURL;
}

function fullpath_keypress(e)
{
  if (e.keyCode == 13)
  {
    var fullpath = document.getElementById("fullpath-text");
    fillListBox(fullpath.value);
  }
}

function togglelist_click()
{
  var fileListBox = document.getElementById("filelist-splitter");
  if (prefs.getListCollapsed())
    fileListBox.setAttribute('state', 'open');
  else
    fileListBox.setAttribute('state', 'collapsed');
  prefs.setListCollapsed( !prefs.getListCollapsed() );
}

function save_click()
{
  var saveasStr = mozImageBundle.formatStringFromName("saveas", [], 0);
  var fileUtil = new FileUtils();
  var mainImage = document.getElementById("file-listbox").selectedItem.lastChild.firstChild;
  //var mainImage = document.getElementById("main-image");
  var nsIFilePicker = Components.interfaces.nsIFilePicker;
  var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);

  fp.init(window, saveasStr, nsIFilePicker.modeSave);  
  fp.appendFilters(nsIFilePicker.filterImages);
  fp.defaultString = baseNameFromUrl(mainImage.src);
  var res = fp.show();
  if (res == nsIFilePicker.returnOK)
  {
    var destFile = fp.file;
    try {
      save(mainImage.src, destFile.path);
    } catch(e) { alert(e); }
  }
}

function saveall_click()
{
  var choosedirStr = mozImageBundle.formatStringFromName("choosedir", [], 0);
  var fileUtil = new FileUtils();
  var basePath = document.getElementById("fullpath-text");
  var filelistbox = document.getElementById("file-listbox");
  var item = null;
  var fileName = "";
  var destFile = "";

  var nsIFilePicker = Components.interfaces.nsIFilePicker;
  var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);

  fp.init(window, choosedirStr, nsIFilePicker.modeGetFolder);  
  fp.appendFilters(nsIFilePicker.filterImages);

  var res = fp.show();
  if (res == nsIFilePicker.returnOK)
  {
    var destDir = fp.file;
    var uri = "";
    var numOfImages = filelistbox.getRowCount();
    try {
      for (var i = 0 ; i < numOfImages ; i++)
      {
        item = filelistbox.getItemAtIndex(i);
        fileName = item.lastChild.lastChild.value;
        if (fileName.substr(0, 7) != "http://")
          uri = pathToURI(fileUtil.append(basePath.value, fileName));
        else
          uri = fileName;
      
        destFile = fileUtil.append(destDir.path, baseNameFromUrl(uri));
        save(uri, destFile);
      }
    } 
    catch(e) { alert(e); }
  }
}

function explorer_click()
{
  try {
    var choosedir = mozImageBundle.GetStringFromName("choosedir");
    var nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
    fp.init(window, choosedir, nsIFilePicker.modeGetFolder);
    var res = fp.show();
    if (res == nsIFilePicker.returnOK)
    {
      var thepath = fp.file;
      document.getElementById('fullpath-text').value = thepath.path;
      fillListBox(thepath.path);
    }
  } 
  catch (e) { alert(e); }
}

  ///////////////////////////////////////////////////////////////////////////
  // These functions have been picked here : 
  // http://www.mozdev.org/source/browse/blozom/junkie/content/sidebarinstall.js?rev=1.1&content-type=text/x-cvsweb-markup
  // It is used to use sidebar in Mozilla as in Firefox.
  ///////////////////////////////////////////////////////////////////////////

/* Addpanel Const */

const DEBUG = true; /* set to false to suppress debug messages */
//const PANELS_RDF_FILE  = "UPnls"; /* directory services property to find panels.rdf */

const SIDEBAR_CONTRACTID   = "@mozilla.org/sidebar;1";
const SIDEBAR_CID      = Components.ID("{22117140-9c6e-11d3-aaf1-00805f8a4905}");
const CONTAINER_CONTRACTID = "@mozilla.org/rdf/container;1";
const DIR_SERV_CONTRACTID  = "@mozilla.org/file/directory_service;1"
const STD_URL_CONTRACTID   = "@mozilla.org/network/standard-url;1"
const IO_SERV_CONTRACTID   = "@mozilla.org/network/io-service;1";
const NETSEARCH_CONTRACTID = "@mozilla.org/rdf/datasource;1?name=internetsearch"
const nsIRDFContainer  = Components.interfaces.nsIRDFContainer;
const nsIProperties    = Components.interfaces.nsIProperties;
const nsIFileURL       = Components.interfaces.nsIFileURL;
const nsIRDFRemoteDataSource = Components.interfaces.nsIRDFRemoteDataSource;

/************/

/* Addpanel functions */

function addPanel(aTitle,aContentURL,aCustomizeURL) {
  debug("addPanel() - start");
  var sidebar = new mySidebar();
  sidebar.addPanel(aTitle,aContentURL,aCustomizeURL);
  debug("addPanel() - end");
  return true;
}

function mySidebar() {
  const RDF_CONTRACTID = "@mozilla.org/rdf/rdf-service;1";
  const nsIRDFService = Components.interfaces.nsIRDFService;

  this.rdf = Components.classes[RDF_CONTRACTID].getService(nsIRDFService);
  this.datasource_uri = getSidebarDatasourceURI(PANELS_RDF_FILE);
  debug('datasource_uri is ' + this.datasource_uri);
  this.resource = 'urn:sidebar:current-panel-list';
  this.datasource = this.rdf.GetDataSource(this.datasource_uri);
}

mySidebar.prototype.nc = "http://home.netscape.com/NC-rdf#";

/* decorate prototype to provide ``class'' methods and property accessors */
mySidebar.prototype.addPanel = function (aTitle,aContentURL,aCustomizeURL) { 
  debug("addPanel(" + aTitle + ", " + aContentURL + ", " + aCustomizeURL + ")");

  // Create a "container" wrapper around the current panels to
  // manipulate the RDF:Seq more easily.
  var panel_list = this.datasource.GetTarget(this.rdf.GetResource(this.resource), this.rdf.GetResource(mySidebar.prototype.nc+"panel-list"), true);
  if (panel_list)
    panel_list.QueryInterface(Components.interfaces.nsIRDFResource);
  else
    // Datasource is busted. Start over.
    debug("Sidebar datasource is busted\n");

  var container = Components.classes[CONTAINER_CONTRACTID].createInstance(nsIRDFContainer);
  container.Init(this.datasource, panel_list);

  /* Create a resource for the new panel and add it to the list */
  var panel_resource = this.rdf.GetResource("urn:sidebar:3rdparty-panel:" + aContentURL);
  var panel_index = container.IndexOf(panel_resource);
  if (panel_index != -1) {  // panel is already in list
    /* do something */
    debug("addPanel(): panel already in list");
 
    return 0;
  }

  /* Now make some sidebar-ish assertions about it... */
  this.datasource.Assert(panel_resource, this.rdf.GetResource(this.nc + "title"), this.rdf.GetLiteral(aTitle), true);
  this.datasource.Assert(panel_resource, this.rdf.GetResource(this.nc + "content"), this.rdf.GetLiteral(aContentURL), true);
  if (aCustomizeURL)
    this.datasource.Assert(panel_resource, this.rdf.GetResource(this.nc + "customize"), this.rdf.GetLiteral(aCustomizeURL), true);        
  container.AppendElement(panel_resource);

  // Use an assertion to pass a "refresh" event to all the sidebars.
  // They use observers to watch for this assertion (in sidebarOverlay.js).
  this.datasource.Assert(this.rdf.GetResource(this.resource), this.rdf.GetResource(this.nc + "refresh"), this.rdf.GetLiteral("true"), true);
  this.datasource.Unassert(this.rdf.GetResource(this.resource), this.rdf.GetResource(this.nc + "refresh"), this.rdf.GetLiteral("true"));

  /* Write the modified panels out. */
  this.datasource.QueryInterface(nsIRDFRemoteDataSource).Flush();
  
  return 1;
}

var debug;

/* static functions */
if (DEBUG)
  debug = function (s) { dump("-*- sidebar component: " + s + "\n"); }
else
  debug = function (s) {}

function getSidebarDatasourceURI(panels_file_id) {
  try {
    /* use the fileLocator to look in the profile directory 
     * to find 'panels.rdf', which is the
     * database of the user's currently selected panels. */
    var directory_service = Components.classes[DIR_SERV_CONTRACTID].getService();
    if (directory_service)
      directory_service = directory_service.QueryInterface(Components.interfaces.nsIProperties);

    /* if <profile>/panels.rdf doesn't exist, get will copy
     * bin/defaults/profile/panels.rdf to <profile>/panels.rdf */
    var sidebar_file = directory_service.get(panels_file_id, Components.interfaces.nsIFile);

    if (!sidebar_file.exists()) {
      /* this should not happen, as GetFileLocation() should copy
       * defaults/panels.rdf to the users profile directory */
      debug("sidebar file does not exist");
      return null;
    }

    if (Components.classes[IO_SERV_CONTRACTID] && Components.interfaces.nsIIOService) { 
         // new code to get URI
         var io_service = Components.classes[IO_SERV_CONTRACTID].getService(Components.interfaces.nsIIOService);
         var file_handler = io_service.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);
         if (file_handler.getURLSpecFromFile) {
             var sidebar_uri = file_handler.getURLSpecFromFile(sidebar_file);
             return sidebar_uri;
         }
    }

    // otherwise try old version
    var file_url = Components.classes[STD_URL_CONTRACTID].createInstance(Components.interfaces.nsIFileURL);
    file_url.file = sidebar_file;

    debug("sidebar uri is " + file_url.spec);
    return file_url.spec;
  } catch (ex) {
    /* this should not happen */
    debug("caught " + ex + " getting sidebar datasource uri");
    return null;
  }
}

/************/


